/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ShellNotify.h"

ShellNotify::ShellNotify(LPITEMIDLIST pidl, Group* pGroup) : Window("IconDeskShellNotify")
{
	// The folder to watch
	_pidl = pidl;

	// The group to notify
	_pGroup = pGroup;

	// Get some window handles
	_hDesktop = FindWindow("DesktopBackgroundClass", NULL);

	// OS variables
	OSVERSIONINFO verInfo;
	verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&verInfo);
	_bWinNT = (verInfo.dwPlatformId == VER_PLATFORM_WIN32_NT);

	// Bind ShellNotify functions
	SHChangeNotifyRegister = (ULONG (WINAPI*)(HWND,int,LONG,UINT,int,SHChangeNotifyEntry*)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), "SHChangeNotifyRegister");
	if(!SHChangeNotifyRegister) {
		DWORD dw = GetLastError();
		LSLogPrintf(LOG_ERROR, "IconDesk", "Could not bind SHChangeNotifyRegister (Error: %u)", dw);
	}
	SHChangeNotifyDeregister = (BOOL (WINAPI*)(ULONG)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), "SHChangeNotifyDeregister");
	if(!SHChangeNotifyDeregister) {
		DWORD dw = GetLastError();
		LSLogPrintf(LOG_ERROR, "IconDesk", "Could not bind SHChangeNotifyDeregister (Error: %u)", dw);
	}
	SHChangeNotification_Lock = (HANDLE (WINAPI *)(HANDLE,DWORD,LPITEMIDLIST**,LONG*)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), "SHChangeNotification_Lock");
	if(!SHChangeNotification_Lock) {
		DWORD dw = GetLastError();
		LSLogPrintf(LOG_ERROR, "IconDesk", "Could not bind SHChangeNotification_Lock (Error: %u)", dw);
	}
	SHChangeNotification_Unlock = (BOOL (WINAPI *)(HANDLE)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), "SHChangeNotification_Unlock");
	if(!SHChangeNotification_Unlock) {
		DWORD dw = GetLastError();
		LSLogPrintf(LOG_ERROR, "IconDesk", "Could not bind SHChangeNotification_Unlock (Error: %u)", dw);
	}

	// Create the window
	if(!createWindow(WS_EX_TOOLWINDOW, "IconDeskShellNotify", WS_CHILD, 0, 0, 0, 0, _hDesktop)) {
		MessageBox(NULL, "Error creating window", "IconDeskShellNotify", MB_OK);
		return;
	}
}

ShellNotify::~ShellNotify()
{
	// Destroy the window
	destroyWindow();
}

void ShellNotify::windowProc(Message& message)
{
	BEGIN_MESSAGEPROC
		MESSAGE(onCreate, WM_CREATE)
		MESSAGE(onDestroy, WM_DESTROY)
		MESSAGE(onEndSession, WM_ENDSESSION)
		MESSAGE(onEndSession, WM_QUERYENDSESSION)
		MESSAGE(onSysCommand, WM_SYSCOMMAND)
		MESSAGE(onShellNotify, WM_APP_SHELLNOTIFY)
	END_MESSAGEPROC
}

void ShellNotify::onCreate(Message& message)
{
	// Hide from VWM
	SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

	SHChangeNotifyEntry lpItems[1];
	lpItems[0].pidl = MakeCopy(_pidl);
	lpItems[0].fRecursive = FALSE;
	_hNotify = SHChangeNotifyRegister(hWnd, SHCNF_INTERRUPT | SHCNF_SHELL | SHCNF_NO_PROXY, SHCNE_ALLEVENTS | SHCNE_INTERRUPT, WM_APP_SHELLNOTIFY, 1, &lpItems[0]);
	if(!_hNotify) {
		LSLog(LOG_DEBUG, "IconDesk", "Could not register shell notifications");
	}
	else {
		char szTitle[MAX_PATH];
		SHGetPathFromIDList(lpItems[0].pidl, szTitle);
		LSLogPrintf(LOG_DEBUG, "IconDesk", "Shell notification registred. PIDL: %d '%s'", lpItems[0].pidl, szTitle);
	}
}

void ShellNotify::onDestroy(Message& message)
{
	if(_hNotify) {
		SHChangeNotifyDeregister(_hNotify);
	}
	LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onDestroy");
}

void ShellNotify::onEndSession(Message& message)
{
	// Forward to parent
	message.lResult = SendMessage(hParent, message.uMsg, message.wParam, message.lParam);
}

void ShellNotify::onSysCommand(Message& message)
{
	if (message.wParam == SC_CLOSE)
		PostMessage(hParent, WM_KEYDOWN, LM_SHUTDOWN, 0);
	else
		message.lResult = DefWindowProc(hWnd, message.uMsg, message.wParam, message.lParam);
}

void ShellNotify::onShellNotify(Message& message)
{
	LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify");

	LPITEMIDLIST *pidls;
	LONG wEventId;
	if(_bWinNT) {
		HANDLE hLock;
		hLock = SHChangeNotification_Lock((HANDLE)message.wParam, (DWORD)message.lParam, &pidls, &wEventId);
		if(hLock) {
			processNotification(wEventId, pidls[0], pidls[1]);
			SHChangeNotification_Unlock(hLock);
		}
	}
	else {
		pidls = (LPITEMIDLIST*)message.wParam;
		wEventId = (LONG)message.lParam;
		processNotification(wEventId, pidls[0], pidls[1]);
	}
}

void ShellNotify::processNotification(LONG wEvent, LPITEMIDLIST pidl1, LPITEMIDLIST pidl2)
{
	switch(wEvent) {
		case SHCNE_RENAMEITEM:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_RENAMEITEM");
			break;
		case SHCNE_CREATE:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_CREATE");
			break;
		case SHCNE_DELETE:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_DELETE");
			break;
		case SHCNE_MKDIR:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_MKDIR");
			break;
		case SHCNE_RMDIR:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_RMDIR");
			break;
		case SHCNE_MEDIAINSERTED:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_MEDIAINSERTED");
			break;
		case SHCNE_MEDIAREMOVED:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_MEDIAREMOVED");
			break;
		case SHCNE_DRIVEREMOVED:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_DRIVEREMOVED");
			break;
		case SHCNE_DRIVEADD:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_DRIVEADD");
			break;
		case SHCNE_NETSHARE:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_NETSHARE");
			break;
		case SHCNE_NETUNSHARE:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_NETUNSHARE");
			break;
		case SHCNE_ATTRIBUTES:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_ATTRIBUTES");
			break;
		case SHCNE_UPDATEDIR:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_UPDATEDIR");
			break;
		case SHCNE_UPDATEITEM:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_UPDATEITEM");
			break;
		case SHCNE_SERVERDISCONNECT:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_SERVERDISCONNECT");
			break;
		case SHCNE_UPDATEIMAGE:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_UPDATEIMAGE");
			break;
		case SHCNE_DRIVEADDGUI:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_DRIVEADDGUI");
			break;
		case SHCNE_RENAMEFOLDER:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_RENAMEFOLDER");
			break;
		case SHCNE_FREESPACE:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify SHCNE_FREESPACE");
			break;
		default:
			LSLog(LOG_DEBUG, "IconDesk", "ShellNotify::onShellNotify unrecognised");
	};

	char szTitle[MAX_PATH];
	SHGetPathFromIDList(pidl1, szTitle);
	LSLogPrintf(LOG_DEBUG, "IconDesk", "UpdateNotification PIDL: %d Path: '%s'", pidl1, szTitle);

	_pGroup->update();
}
